@@ -16,7 +16,7 @@ import inspect
 import os
 import random
 from collections import ChainMap
-from typing import Any, Dict, Iterable, List, Optional, Set, Union
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Set, Union
 
 import numpy as np
 import tensorflow as tf
@@ -25,10 +25,9 @@ from tensorflow.python.distribute.input_lib import DistributedDataset
 from torch.utils.data import DataLoader
 
 import fastestimator as fe
-from fastestimator.backend.to_shape import to_shape
-from fastestimator.backend.to_tensor import to_tensor
-from fastestimator.backend.to_type import to_type
-from fastestimator.dataset.op_dataset import OpDataset
+from fastestimator.backend._to_shape import to_shape
+from fastestimator.backend._to_tensor import to_tensor
+from fastestimator.backend._to_type import to_type
 from fastestimator.network import BaseNetwork, TFNetwork, TorchNetwork
 from fastestimator.pipeline import Pipeline
 from fastestimator.schedule.schedule import Scheduler, get_current_items, get_signature_epochs
@@ -40,9 +39,21 @@ from fastestimator.trace.io.restore_wizard import RestoreWizard
 from fastestimator.trace.io.traceability import Traceability
 from fastestimator.trace.trace import EvalEssential, Logger, PerDSTrace, TestEssential, Trace, TrainEssential, \
     sort_traces
-from fastestimator.util.data import Data
+from fastestimator.util.base_util import NonContext, Suppressor, to_list, to_set
+from fastestimator.util.data import Data, FilteredData
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import NonContext, Suppressor, draw, to_list, to_set
+from fastestimator.util.util import draw
+
+
+def _verify_dependency_versions() -> None:
+    """Print warning messages if the user is using unexpected versions of TF or torch.
+    """
+    if tf.__version__ != '2.9.1':
+        print("\033[93m{}\033[00m".format("FastEstimator-Warn: Expected TensorFlow version 2.9.1 but found "
+                                          f"{tf.__version__}. The framework may not work as expected."))
+    if torch.__version__ not in ('1.10.2', '1.10.2+cpu', '1.10.2+cu113'):
+        print("\033[93m{}\033[00m".format("FastEstimator-Warn: Expected PyTorch version 1.10.2 but found "
+                                          f"{torch.__version__}. The framework may not work as expected."))
 
 
 @traceable()
@@ -53,17 +64,23 @@ class Estimator:
     (estimator.fit) or test (estimator.test) models. It wraps `Pipeline`, `Network`, `Trace` objects together and
     defines the whole optimization process.
 
+    If the data fed into pipeline is a TensorFlow Dataset, then the parameters `train_steps_per_epoch` and
+    `eval_steps_per_epoch` can only reduce the number of steps per epoch. If these parameters are higher than the
+    dimension of the stated Dataset then the whole Dataset will be used.
+
+
     Args:
         pipeline: An fe.Pipeline object that defines the data processing workflow.
         network: An fe.Network object that contains models and other training graph definitions.
         epochs: The number of epochs to run.
-        max_train_steps_per_epoch: Training will complete after n steps even if loader is not yet exhausted. If None,
-            all data will be used.
-        max_eval_steps_per_epoch: Evaluation will complete after n steps even if loader is not yet exhausted. If None,
-            all data will be used.
+        train_steps_per_epoch: Training will be cut short or extended to complete N steps even if loader is not yet
+            exhausted. If None, all data will be used.
+        eval_steps_per_epoch: Evaluation will be cut short or extended to complete N steps even if loader is not yet
+            exhausted. If None, all data will be used.
         traces: What Traces to run during training. If None, only the system's default Traces will be included.
         log_steps: Frequency (in steps) for printing log messages. 0 to disable all step-based printing (though epoch
             information will still print). None to completely disable printing.
+        eval_log_steps: The list of steps on which evaluation progress logs need to be printed.
         monitor_names: Additional keys from the data dictionary to be written into the logs.
     """
     monitor_names: Set[str]
@@ -75,10 +92,11 @@ class Estimator:
                  pipeline: Pipeline,
                  network: BaseNetwork,
                  epochs: int,
-                 max_train_steps_per_epoch: Optional[int] = None,
-                 max_eval_steps_per_epoch: Optional[int] = None,
+                 train_steps_per_epoch: Optional[int] = None,
+                 eval_steps_per_epoch: Optional[int] = None,
                  traces: Union[None, Trace, Scheduler[Trace], Iterable[Union[Trace, Scheduler[Trace]]]] = None,
                  log_steps: Optional[int] = 100,
+                 eval_log_steps: Sequence[int] = (),
                  monitor_names: Union[None, str, Iterable[str]] = None):
         self.traces_in_use = []
         self.filepath = os.path.realpath(inspect.stack()[2].filename)  # Record this for history tracking
@@ -90,8 +108,9 @@ class Estimator:
                              traces=to_list(traces),
                              log_steps=log_steps,
                              total_epochs=epochs,
-                             max_train_steps_per_epoch=max_train_steps_per_epoch,
-                             max_eval_steps_per_epoch=max_eval_steps_per_epoch,
+                             train_steps_per_epoch=train_steps_per_epoch,
+                             eval_steps_per_epoch=eval_steps_per_epoch,
+                             eval_log_steps=eval_log_steps,
                              system_config=self.fe_summary())
 
     @property
@@ -122,6 +141,8 @@ class Estimator:
         Returns:
             A summary object containing the training history for this session iff a `summary` name was provided.
         """
+        _verify_dependency_versions()
+        os.environ['TF_CPP_MIN_LOG_LEVEL'] = '1'  # Prevent tf from constantly printing useless information
         draw()
         self.system.reset(summary, self.fe_summary())
         self._prepare_traces(run_modes={"train", "eval"})
@@ -178,6 +199,8 @@ class Estimator:
             A summary object containing the training history for this session iff the `summary` name is not None (after
             considering the default behavior above).
         """
+        _verify_dependency_versions()
+        os.environ['TF_CPP_MIN_LOG_LEVEL'] = '1'  # Prevent tf from constantly printing useless information
         self.system.reset_for_test(summary)
         self._prepare_traces(run_modes={"test"})
         self._start(run_modes={"test"}, eager=eager)
@@ -219,6 +242,7 @@ class Estimator:
                     with self.pipeline(mode=mode,
                                        epoch=epoch,
                                        ds_id=ds_id,
+                                       steps_per_epoch=None,
                                        output_keys=trace_input_keys - network_output_keys
                                        | network_input_keys) as loader:
                         loader = self._configure_loader(loader)
@@ -325,10 +349,20 @@ class Estimator:
                                     ds_id=self.system.ds_id,
                                     output_keys=trace_input_keys,
                                     eager=eager)
+
             with self.pipeline(mode=self.system.mode,
                                epoch=self.system.epoch_idx,
                                ds_id=self.system.ds_id,
+                               steps_per_epoch=self.system.steps_per_epoch,
                                output_keys=trace_input_keys - network_output_keys | network_input_keys) as loader:
+
+                if self.system.mode == 'eval':
+                    log_steps_per_epoch = len(loader) // loader.get_batch_size(
+                    ) if not self.system.steps_per_epoch else self.system.steps_per_epoch
+                    self.system.eval_log_steps = [
+                        1, log_steps_per_epoch // 3, (2 * log_steps_per_epoch) // 3, log_steps_per_epoch
+                    ] if not self.system.eval_log_steps else self.system.eval_log_steps
+
                 loader = self._configure_loader(loader)
                 iterator = iter(loader)
                 with Suppressor():
@@ -345,13 +379,13 @@ class Estimator:
                         self.system.update_batch_idx()
                         batch = self._configure_tensor(loader, batch)
                         self._run_traces_on_batch_begin(batch, traces=ds_traces)
+
                         batch, prediction = self.network.run_step(batch)
                         self._run_traces_on_batch_end(batch, prediction, traces=ds_traces)
-                        if isinstance(loader,
-                                      DataLoader) and ((self.system.batch_idx == self.system.max_train_steps_per_epoch
-                                                        and self.system.mode == "train") or
-                                                       (self.system.batch_idx == self.system.max_eval_steps_per_epoch
-                                                        and self.system.mode == "eval")):
+                        if isinstance(loader, DataLoader) and (
+                            (self.system.batch_idx == self.system.train_steps_per_epoch and self.system.mode == "train")
+                                or
+                            (self.system.batch_idx == self.system.eval_steps_per_epoch and self.system.mode == "eval")):
                             raise StopIteration
                         with Suppressor():
                             batch = next(iterator)
@@ -373,21 +407,37 @@ class Estimator:
         Returns:
             The potentially modified dataloader to be used for training.
         """
+
         new_loader = loader
         if isinstance(new_loader, DataLoader) and isinstance(self.network, TFNetwork):
-            add_batch = True
-            if isinstance(loader.dataset, OpDataset) and loader.dataset.dataset.fe_batch:
+            add_batch = bool(new_loader.batch_size)
+            if hasattr(loader, 'fe_postprocess_fn') and loader.fe_postprocess_fn is not None:
+                # The user is manually batching data and running ops on data batches. No reliable way to shortcut this
+                # since ops might require specific batch composition.
+                data_instance = next(iter(loader))
                 add_batch = False
-            batch = to_tensor(loader.dataset[0], target_type="tf")
-            data_type = to_type(batch)
-            data_shape = to_shape(batch, add_batch=add_batch, exact_shape=False)
+            else:
+                # No batch-based ops so we can try and just use the OpDataset to more quickly get our data summary
+                data_instance = loader.dataset[0]
+                if isinstance(data_instance, list):
+                    # This is a batched dataset
+                    data_instance = data_instance[0]
+                    add_batch = True
+                if isinstance(data_instance, FilteredData):
+                    # We got unlucky and drew filtered data as the zeroth element. Fall back to a slower but more robust
+                    # analysis of the batch
+                    data_instance = next(iter(loader))
+                    add_batch = False
+            data_instance = to_tensor(data_instance, target_type="tf")
+            data_type = to_type(data_instance)
+            data_shape = to_shape(data_instance, add_batch=add_batch, exact_shape=False)
             new_loader = tf.data.Dataset.from_generator(lambda: loader, data_type, output_shapes=data_shape)
             new_loader = new_loader.prefetch(1)
         if isinstance(new_loader, tf.data.Dataset):
-            if self.system.max_train_steps_per_epoch and self.system.mode == "train":
-                new_loader = new_loader.take(self.system.max_train_steps_per_epoch)
-            if self.system.max_eval_steps_per_epoch and self.system.mode == "eval":
-                new_loader = new_loader.take(self.system.max_eval_steps_per_epoch)
+            if self.system.train_steps_per_epoch and self.system.mode == "train":
+                new_loader = new_loader.take(self.system.train_steps_per_epoch)
+            if self.system.eval_steps_per_epoch and self.system.mode == "eval":
+                new_loader = new_loader.take(self.system.eval_steps_per_epoch)
             if isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy) and isinstance(
                     self.network, TFNetwork) and not isinstance(new_loader, DistributedDataset):
                 # The default autoshard policy is file, changing it to data to avoid warning
@@ -403,6 +453,7 @@ class Estimator:
         Returns:
             Either the original `batch`, or the `batch` converted to torch.Tensors if required.
         """
+        # TODO - if user has torch loader but custom collate that doesn't return torch tensor, need to cast here
         if isinstance(loader, tf.data.Dataset) and isinstance(self.network, TorchNetwork):
             batch = to_tensor(batch, target_type="torch")
         return batch
@@ -552,6 +603,8 @@ def enable_deterministic(seed: int) -> None:
     np.random.seed(seed)
     tf.random.set_seed(seed)
     torch.manual_seed(seed)
+    tf.keras.utils.set_random_seed(seed)
+    tf.config.experimental.enable_op_determinism()
 
 
 def record_history(path: Union[bool, str]) -> None:
