@@ -12,5 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from fastestimator.layers.pytorch.cropping_2d import Cropping2D
-from fastestimator.layers.pytorch.hadamard import HadamardCode
+from typing import TYPE_CHECKING
+
+import lazy_loader as lazy
+
+__getattr__, __dir__, __all__ = lazy.attach(__name__,
+                                            submod_attrs={'cropping_2d': ['Cropping2D'],
+                                                          'hadamard': ['HadamardCode'],
+                                                          })
+
+if TYPE_CHECKING:
+    from fastestimator.layers.pytorch.cropping_2d import Cropping2D
+    from fastestimator.layers.pytorch.hadamard import HadamardCode
