@@ -14,6 +14,7 @@
 # ==============================================================================
 import gc
 import os
+import sys
 import tempfile
 from collections import ChainMap
 from typing import Any, Callable, Dict, Iterable, List, MutableMapping, Optional, Set, Tuple, TypeVar, Union
@@ -24,16 +25,19 @@ import tensorflow as tf
 import tensorflow.keras.mixed_precision as mixed_precision_tf
 import torch
 from tensorflow.python.distribute.values import DistributedValues
+from tensorflow.python.keras.engine.sequential import Sequential
 
-from fastestimator.backend.load_model import load_model
-from fastestimator.backend.to_tensor import to_tensor
-from fastestimator.op.numpyop import NumpyOp, forward_numpyop
+import fastestimator as fe
+from fastestimator.backend._load_model import load_model
+from fastestimator.backend._to_tensor import to_tensor
+from fastestimator.op.numpyop import Batch, NumpyOp, RemoveIf, forward_numpyop
 from fastestimator.op.op import get_inputs_by_op, write_outputs_by_op
 from fastestimator.op.tensorop.model.update import UpdateOp
 from fastestimator.op.tensorop.tensorop import TensorOp
 from fastestimator.schedule.schedule import EpochScheduler, RepeatScheduler, Scheduler, get_current_items
+from fastestimator.util.base_util import NonContext, to_list
 from fastestimator.util.traceability_util import trace_model, traceable
-from fastestimator.util.util import NonContext, get_batch_size, to_list
+from fastestimator.util.util import get_batch_size
 
 Model = TypeVar('Model', tf.keras.Model, torch.nn.Module)
 T = TypeVar('T')
@@ -71,6 +75,11 @@ class BaseNetwork:
             op.build(framework=self.target_type, device=self.device)
         self.models = to_list(_collect_models(ops))
         self.postprocessing = to_list(postprocessing)
+        for pop in self.postprocessing:
+            if isinstance(pop, RemoveIf):
+                raise ValueError("Filtering is currently not supported in network post-processing")
+            if isinstance(pop, Batch):
+                raise ValueError("Post-processing data is already batched, so Batch Op is not supported here.")
         self._verify_inputs()
         self.effective_inputs = dict()
         self.effective_outputs = dict()
@@ -112,7 +121,7 @@ class BaseNetwork:
     def load_epoch(self,
                    mode: str,
                    epoch: int,
-                   ds_id: Union[str, None],
+                   ds_id: str,
                    output_keys: Optional[Set[str]] = None,
                    warmup: bool = False,
                    eager: bool = False) -> None:
@@ -124,7 +133,7 @@ class BaseNetwork:
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
             output_keys: What keys can be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
             eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
@@ -173,13 +182,13 @@ class BaseNetwork:
             loss_keys |= op.get_fe_loss_keys()
         return loss_keys
 
-    def get_effective_input_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
+    def get_effective_input_keys(self, mode: str, epoch: int, ds_id: str = '') -> Set[str]:
         """Determine which keys need to be provided as input to the network during the given `epoch`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider for determining inputs.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
 
         Returns:
             The necessary inputs for the network to execute the given `epoch` and `mode`.
@@ -191,13 +200,13 @@ class BaseNetwork:
             produced_keys.update(op.outputs)
         return input_keys
 
-    def _get_effective_postprocessing_input_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
+    def _get_effective_postprocessing_input_keys(self, mode: str, epoch: int, ds_id: str = '') -> Set[str]:
         """Determine which keys need to be provided as input to the postprocessing during the given `epoch`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider for determining inputs.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
 
         Returns:
             The necessary inputs for the postprocessing to execute the given `epoch` and `mode`.
@@ -209,13 +218,13 @@ class BaseNetwork:
             produced_keys.update(op.outputs)
         return input_keys
 
-    def get_all_output_keys(self, mode: str, epoch: int, ds_id: Optional[str] = None) -> Set[str]:
+    def get_all_output_keys(self, mode: str, epoch: int, ds_id: str = '') -> Set[str]:
         """Get all of the keys that will be generated by the network during the given `epoch` and `mode`.
 
         Args:
             mode: The execution mode to consider. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch number to consider when searching for outputs.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
 
         Returns:
             The keys that will be generated by the network's Ops during the `epoch` for the given `mode`.
@@ -260,7 +269,7 @@ class BaseNetwork:
         forward_numpyop(ops=self.epoch_postprocessing,
                         data=ChainMap(prediction, batch),
                         state=self.epoch_state,
-                        batched=True)
+                        batched=self.target_type)
         return batch, prediction
 
     def _run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:  # Batch, Prediction
@@ -277,15 +286,14 @@ class BaseNetwork:
         """
         raise NotImplementedError
 
-    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: Optional[
-                                                                                      str] = None) -> Dict[str, Any]:
+    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: str = '') -> Dict[str, Any]:
         """Run a forward step through the Network on an element of data.
 
         Args:
             data: The element to data to use as input.
             mode: The mode in which to run the transform. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch in which to run the transform.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
 
         Returns:
             prediction_data overlaid on the input `data`.
@@ -385,7 +393,7 @@ class TorchNetwork(BaseNetwork):
     def load_epoch(self,
                    mode: str,
                    epoch: int,
-                   ds_id: Union[str, None],
+                   ds_id: str,
                    output_keys: Optional[Set[str]] = None,
                    warmup: bool = False,
                    eager: bool = False) -> None:
@@ -397,7 +405,7 @@ class TorchNetwork(BaseNetwork):
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
             output_keys: What keys must be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
             eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
@@ -433,7 +441,7 @@ class TorchNetwork(BaseNetwork):
             else:
                 try:
                     data[key] = data[key].to(device)
-                except:
+                except (RuntimeError, AssertionError, AttributeError):
                     pass
 
     def unload_epoch(self) -> None:
@@ -574,7 +582,7 @@ class TFNetwork(BaseNetwork):
     def load_epoch(self,
                    mode: str,
                    epoch: int,
-                   ds_id: Union[None, str],
+                   ds_id: str,
                    output_keys: Optional[Set[str]] = None,
                    warmup: bool = False,
                    eager: bool = False) -> None:
@@ -586,7 +594,7 @@ class TFNetwork(BaseNetwork):
         Args:
             mode: The mode to prepare to execute. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch to prepare to execute.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
             output_keys: What keys must be moved from the GPU back to the CPU after executing a step.
             warmup: Whether to prepare to execute it warmup mode or not (end users can likely ignore this argument).
             eager: Whether to run the training in eager mode. This is only related to TensorFlow training because
@@ -595,10 +603,20 @@ class TFNetwork(BaseNetwork):
         super().load_epoch(mode=mode, epoch=epoch, ds_id=ds_id, output_keys=output_keys, warmup=warmup, eager=eager)
         # Don't cause a re-trace just because epoch changed
         self.epoch_state["epoch"] = tf.convert_to_tensor(self.epoch_state["epoch"])
-        # Need to re-trace the TF graph if the optimizer is changing due to scheduling:
+        # Need to re-trace the TF graph if optimizer or layer trainable setting is changing:
+        trainable_str = "".join([str(layer.trainable) for model in self.epoch_models for layer in model.layers])
         opt_str = "x".join(
             [str(id(model.current_optimizer)) for model in self.epoch_models if hasattr(model, 'current_optimizer')])
-        self.epoch_state["_force_tf_retrace"] = hash(opt_str)  # Hash to keep at fixed memory overhead
+        self.epoch_state["_force_tf_retrace"] = hash(trainable_str + opt_str)  # Hash to keep at fixed memory overhead
+
+    def unload_epoch(self) -> None:
+        # This prevents a tf graph memory leak that would slow down long trainings. Since we
+        # re-build graphs every epoch there is no reason to keep old ones around.
+        strategy = tf.distribute.get_strategy()
+        if isinstance(strategy, tf.distribute.MirroredStrategy):
+            return  # TODO - Find a way to clear graph for multi-gpu
+        else:
+            tf.keras.backend.clear_session()
 
     def _run_step(self, batch: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
         """Run a forward step through the Network on a batch of data.
@@ -715,7 +733,7 @@ class TFNetwork(BaseNetwork):
                 prediction[key] = batch[key]
         return prediction
 
-    @tf.function(experimental_relax_shapes=True)
+    @tf.function(reduce_retracing=True)
     def _forward_step_static(self,
                              batch: Dict[str, Any],
                              state: Dict[str, Any],
@@ -745,14 +763,14 @@ class TFNetwork(BaseNetwork):
                 prediction[key] = batch[key]
         return prediction
 
-    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: Optional[str] = None) -> Dict[str, Any]:
+    def transform(self, data: Dict[str, Any], mode: str, epoch: int = 1, ds_id: str = '') -> Dict[str, Any]:
         """Run a forward step through the Network on an element of data.
 
         Args:
             data: The element to data to use as input.
             mode: The mode in which to run the transform. One of 'train', 'eval', 'test', or 'infer'.
             epoch: The epoch in which to run the transform.
-            ds_id: The current dataset id. If None, ops with all ds_id will be considered.
+            ds_id: The current dataset id.
 
         Returns:
             (batch_data, prediction_data)
@@ -856,12 +874,11 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
         models: The model(s) built by FastEstimator.
     """
     def _generate_model_names(num_names):
-        names = ["model" if i + build.count == 0 else "model{}".format(i + build.count) for i in range(num_names)]
-        build.count += num_names
+        names = ["model" if i + fe.fe_build_count == 0 else "model{}".format(i + fe.fe_build_count) for i in
+                 range(num_names)]
+        fe.fe_build_count += num_names
         return names
 
-    if not hasattr(build, "count"):
-        build.count = 0
     # The following garbage collection is needed for if a TF model was running, but then died due to an exception being
     # thrown, but the exception was then caught, whereupon the user wanted to switch to a pytorch model instead. Absent
     # this collection, you would see: "Failed setting context: CUDA_ERROR_NOT_INITIALIZED: initialization error". This
@@ -872,13 +889,27 @@ def build(model_fn: Callable[[], Union[Model, List[Model]]],
     # tensorflow models requires setting global policies prior to model creation. Since there is no way to know the
     # framework of model, setting the policy for both tf and pytorch here.
     if mixed_precision:
-        mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('mixed_float16'))
+        if sys.platform == 'darwin':
+            print("\033[93m{}\033[00m".format("FastEstimator-Warn: Mixed Precision is not currently supported on Mac / "
+                                              "Metal. This flag will be ignored."))
+            mixed_precision = False
+        else:
+            mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('mixed_float16'))
     else:
         mixed_precision_tf.set_global_policy(mixed_precision_tf.Policy('float32'))
+    models = None
     if torch.cuda.device_count() > 1:
+        # We need to figure out whether model_fn returns tf models or torch models
         if not isinstance(tf.distribute.get_strategy(), tf.distribute.MirroredStrategy):
-            tf.distribute.experimental_set_strategy(tf.distribute.MirroredStrategy())
-    models, optimizer_fn = to_list(model_fn()), to_list(optimizer_fn)
+            # If we've already done this and gotten TF model, the above flag will be set and this will be skipped. If we
+            # are dealing with pytorch models, the model_fn() invocation will be kept so as to not waste clock cycles.
+            models = to_list(model_fn())
+            if isinstance(models[0], tf.keras.Model):
+                models = None  # We will re-instantiate the models again now that we know we need MirroredStrategy
+                tf.keras.backend.clear_session()  # This will reset the automatic layer naming in case user is
+                # extracting intermediate layer outputs by name
+                tf.distribute.experimental_set_strategy(tf.distribute.MirroredStrategy())
+    models, optimizer_fn = to_list(model_fn()) if models is None else models, to_list(optimizer_fn)
     # fill optimizers if optimizer_fn is None
     if not optimizer_fn:
         optimizer_fn = [None] * len(models)
@@ -927,6 +958,9 @@ def _fe_compile(model: Model,
         framework = "tf"
     elif isinstance(model, torch.nn.Module):
         framework = "torch"
+    elif isinstance(model, Sequential):
+        raise DeprecationWarning("Importing from tensorflow.python.keras.models/layers is deprecated. Import from "
+                                 "tensorflow.keras.models/layers instead")
     else:
         raise ValueError("unrecognized model format: {}".format(type(model)))
     # torch multi-gpu handling
