@@ -12,5 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from fastestimator.op import numpyop, tensorop
-from fastestimator.op.op import Op, get_inputs_by_op, write_outputs_by_op
+from typing import TYPE_CHECKING
+
+import lazy_loader as lazy
+
+__getattr__, __dir__, __all__ = lazy.attach(__name__,
+                                            submodules={'numpyop', 'tensorop'},
+                                            submod_attrs={'op': ['Op', 'get_inputs_by_op', 'write_outputs_by_op']})
+
+if TYPE_CHECKING:
+    from fastestimator.op import numpyop, tensorop
+    from fastestimator.op.op import Op, get_inputs_by_op, write_outputs_by_op
