@@ -17,9 +17,9 @@ from typing import Any, Dict, List, Union
 
 import numpy as np
 
-from fastestimator.op.numpyop.numpyop import NumpyOp, forward_numpyop
+from fastestimator.op.numpyop.numpyop import Batch, Delete, NumpyOp, forward_numpyop
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import to_list
+from fastestimator.util.base_util import to_list
 
 
 @traceable()
@@ -42,12 +42,16 @@ class Fuse(NumpyOp):
         mode = ops[0].mode
         ds_id = ops[0].ds_id
         for op in ops:
+            if isinstance(op, Batch):
+                raise ValueError("Cannot nest the Batch op inside of Fuse")
             if op.mode != mode:
                 raise ValueError(f"All Fuse ops must share the same mode, but got {mode} and {op.mode}")
             if op.ds_id != ds_id:
                 raise ValueError(f"All Fuse ops must share the same ds_id, but got {ds_id} and {op.ds_id}")
             for inp in op.inputs:
-                if inp not in inputs and inp not in outputs:
+                if isinstance(op, Delete) and inp in outputs:
+                    outputs.remove(inp)
+                elif inp not in inputs and inp not in outputs:
                     inputs.append(inp)
             for out in op.outputs:
                 if out not in outputs:
@@ -79,5 +83,11 @@ class Fuse(NumpyOp):
 
     def forward(self, data: List[np.ndarray], state: Dict[str, Any]) -> List[np.ndarray]:
         data = {key: elem for key, elem in zip(self.inputs, data)}
-        forward_numpyop(self.ops, data, state)
-        return [data[key] for key in self.outputs]
+        filtered = forward_numpyop(self.ops, data, state)
+        return filtered if filtered else [data[key] for key in self.outputs]
+
+    def forward_batch(self, data: Union[np.ndarray, List[np.ndarray]],
+                      state: Dict[str, Any]) -> Union[np.ndarray, List[np.ndarray]]:
+        data = {key: elem for key, elem in zip(self.inputs, data)}
+        filtered = forward_numpyop(self.ops, data, state, batched="np")
+        return filtered if filtered else [data[key] for key in self.outputs]
