@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from typing import List, Optional, Tuple, Union
+from typing import Iterable, List, Optional, Tuple, Union
 
 import cv2
 from albumentations.augmentations.transforms import OpticalDistortion as OpticalDistortionAlb
@@ -29,6 +29,8 @@ class OpticalDistortion(MultiVariateAlbumentation):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
         image_in: The key of an image to be modified.
         mask_in: The key of a mask to be modified (with the same random factors as the image).
         masks_in: The key of masks to be modified (with the same random factors as the image).
@@ -36,7 +38,7 @@ class OpticalDistortion(MultiVariateAlbumentation):
         mask_out: The key to write the modified mask (defaults to `mask_in` if None).
         masks_out: The key to write the modified masks (defaults to `masks_in` if None).
         distort_limit: If distort_limit is a single float, the range will be (-distort_limit, distort_limit).
-        shift_limit: If shift_limit is a single float, the range will be (-shift_limit, shift_limit). 
+        shift_limit: If shift_limit is a single float, the range will be (-shift_limit, shift_limit).
         interpolation: Flag that is used to specify the interpolation algorithm. Should be one of:
             cv2.INTER_NEAREST, cv2.INTER_LINEAR, cv2.INTER_CUBIC, cv2.INTER_AREA, cv2.INTER_LANCZOS4.
         border_mode: Flag that is used to specify the pixel extrapolation method. Should be one of:
@@ -54,7 +56,8 @@ class OpticalDistortion(MultiVariateAlbumentation):
                  border_mode: int = cv2.BORDER_REFLECT_101,
                  value: Union[None, int, float, List[int], List[float]] = None,
                  mask_value: Union[None, int, float, List[int], List[float]] = None,
-                 mode: Optional[str] = None,
+                 mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None,
                  image_in: Optional[str] = None,
                  mask_in: Optional[str] = None,
                  masks_in: Optional[str] = None,
@@ -81,4 +84,5 @@ class OpticalDistortion(MultiVariateAlbumentation):
             keypoints_out=None,
             bbox_params=None,
             keypoint_params=None,
-            mode=mode)
+            mode=mode,
+            ds_id=ds_id)
