@@ -31,6 +31,8 @@ class Equalize(ImageOnlyAlbumentation):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
         eq_mode: {'cv', 'pil'}. Use OpenCV or Pillow equalization method.
         by_channels: If True, use equalization by channels separately, else convert image to YCbCr representation and
             use equalization by `Y` channel.
@@ -45,6 +47,7 @@ class Equalize(ImageOnlyAlbumentation):
                  inputs: Union[str, Iterable[str]],
                  outputs: Union[str, Iterable[str]],
                  mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None,
                  eq_mode: str = "cv",
                  by_channels: bool = True,
                  mask: Union[None, np.ndarray] = None,
@@ -53,4 +56,5 @@ class Equalize(ImageOnlyAlbumentation):
             EqualizeAlb(mode=eq_mode, by_channels=by_channels, mask=mask, mask_params=mask_params, always_apply=True),
             inputs=inputs,
             outputs=outputs,
-            mode=mode)
+            mode=mode,
+            ds_id=ds_id)
