@@ -22,7 +22,7 @@ from fastestimator.util.traceability_util import traceable
 
 @traceable()
 class ExpandDims(NumpyOp):
-    """Transpose the data (for example to make it channel-width-height instead of width-height-channel)
+    """Expand the dimension of inputs by inserting a new axis to the specified axis position.
 
     Args:
         inputs: Key(s) of inputs to be modified.
@@ -30,14 +30,17 @@ class ExpandDims(NumpyOp):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
         axis: The axis to expand.
     """
     def __init__(self,
                  inputs: Union[str, Iterable[str]],
                  outputs: Union[str, Iterable[str]],
                  mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None,
                  axis: int = -1):
-        super().__init__(inputs=inputs, outputs=outputs, mode=mode)
+        super().__init__(inputs=inputs, outputs=outputs, mode=mode, ds_id=ds_id)
         self.axis = axis
         self.in_list, self.out_list = True, True
 
