@@ -30,6 +30,8 @@ class ImageCompression(ImageOnlyAlbumentation):
         mode: What mode(s) to execute this Op in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
+        ds_id: What dataset id(s) to execute this Op in. To execute regardless of ds_id, pass None. To execute in all
+            ds_ids except for a particular one, you can pass an argument like "!ds1".
         quality_lower: Lower bound on the image quality. Should be in [0, 100] range for jpeg and [1, 100] for webp.
         quality_upper: Upper bound on the image quality. Should be in [0, 100] range for jpeg and [1, 100] for webp.
         compression_type: should be ImageCompressionType.JPEG or ImageCompressionType.WEBP.
@@ -41,6 +43,7 @@ class ImageCompression(ImageOnlyAlbumentation):
                  inputs: Union[str, Iterable[str]],
                  outputs: Union[str, Iterable[str]],
                  mode: Union[None, str, Iterable[str]] = None,
+                 ds_id: Union[None, str, Iterable[str]] = None,
                  quality_lower: float = 99,
                  quality_upper: float = 100,
                  compression_type: ImgCmpAlb.ImageCompressionType = ImgCmpAlb.ImageCompressionType.JPEG):
@@ -51,4 +54,5 @@ class ImageCompression(ImageOnlyAlbumentation):
                       always_apply=True),
             inputs=inputs,
             outputs=outputs,
-            mode=mode)
+            mode=mode,
+            ds_id=ds_id)
