@@ -17,10 +17,10 @@ from typing import Any, Dict, Iterable, List, Optional, TypeVar, Union
 import tensorflow as tf
 import torch
 
-from fastestimator.backend.get_gradient import get_gradient
+from fastestimator.backend._get_gradient import get_gradient
 from fastestimator.op.tensorop.tensorop import TensorOp
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import to_list
+from fastestimator.util.base_util import to_list
 
 Tensor = TypeVar('Tensor', tf.Tensor, torch.Tensor)
 
@@ -88,7 +88,7 @@ class GradientOp(TensorOp):
             elif self.framework == "torch":
                 trainable_params = [p for p in self.model.parameters() if p.requires_grad]
                 for idx, final in enumerate(finals):
-                    # get_gradinet
+                    # get_gradient
                     retain_graph = self.retain_graph or not idx == len(finals) - 1
                     gradient = get_gradient(final, trainable_params, retain_graph=retain_graph)
                     results.append(gradient)
