@@ -15,12 +15,11 @@
 import os
 import re
 from collections import defaultdict
-from typing import List, Optional, Set, Iterable
+from typing import Iterable, List, Optional, Set
 
-from fastestimator.dataset.dir_dataset import DirDataset
 from fastestimator.summary.logs.log_plot import visualize_logs
 from fastestimator.summary.summary import Summary, ValWithError, average_summaries
-from fastestimator.util.util import strip_suffix
+from fastestimator.util.base_util import list_files, strip_suffix
 
 
 def parse_log_iter(source: Iterable[str], sync: Summary) -> Summary:
@@ -37,11 +36,11 @@ def parse_log_iter(source: Iterable[str], sync: Summary) -> Summary:
     last_epoch = 0
     for line in source:
         mode = None
-        if line.startswith("FastEstimator-Train") or line.startswith("FastEstimator-Finish"):
+        if line.startswith("FastEstimator-Train: step") or line.startswith("FastEstimator-Finish"):
             mode = "train"
-        elif line.startswith("FastEstimator-Eval"):
+        elif line.startswith("FastEstimator-Eval: step"):
             mode = "eval"
-        elif line.startswith("FastEstimator-Test"):
+        elif line.startswith("FastEstimator-Test: step"):
             mode = "test"
         if mode is None:
             continue
@@ -101,7 +100,6 @@ def parse_log_files(file_paths: List[str],
                     save_path: Optional[str] = None,
                     ignore_metrics: Optional[Set[str]] = None,
                     include_metrics: Optional[Set[str]] = None,
-                    share_legend: bool = True,
                     pretty_names: bool = False,
                     group_by: Optional[str] = None) -> None:
     """Parse one or more log files for graphing.
@@ -117,7 +115,6 @@ def parse_log_files(file_paths: List[str],
         save_path: Where to save the image if save is true. Defaults to dir_path if not provided.
         ignore_metrics: Any metrics within the log files which will not be visualized.
         include_metrics: A whitelist of metric keys (None whitelists all keys).
-        share_legend: Whether to have one legend across all graphs (True) or one legend per graph (False).
         pretty_names: Whether to modify the metric names in graph titles (True) or leave them alone (False).
         group_by: Combine multiple log files by a regex to visualize their mean+-stddev. For example, to group together
             files like [a_1.txt, a_2.txt] vs [b_1.txt, b_2.txt] you can use: r'(.*)_[\d]+\.txt'.
@@ -129,7 +126,7 @@ def parse_log_files(file_paths: List[str],
     if file_paths is None or len(file_paths) < 1:
         raise AssertionError("must provide at least one log file")
     if save and save_path is None:
-        save_path = file_paths[0]
+        save_path = os.path.join(os.path.dirname(file_paths[0]), 'parse_logs.html')
 
     groups = defaultdict(list)  # {group_name: [experiment(s)]}
     for path in file_paths:
@@ -144,7 +141,6 @@ def parse_log_files(file_paths: List[str],
     visualize_logs(experiments,
                    save_path=save_path,
                    smooth_factor=smooth_factor,
-                   share_legend=share_legend,
                    pretty_names=pretty_names,
                    ignore_metrics=ignore_metrics,
                    include_metrics=include_metrics)
@@ -153,12 +149,11 @@ def parse_log_files(file_paths: List[str],
 def parse_log_dir(dir_path: str,
                   log_extension: str = '.txt',
                   recursive_search: bool = False,
-                  smooth_factor: float = 1,
+                  smooth_factor: float = 0,
                   save: bool = False,
                   save_path: Optional[str] = None,
                   ignore_metrics: Optional[Set[str]] = None,
                   include_metrics: Optional[Set[str]] = None,
-                  share_legend: bool = True,
                   pretty_names: bool = False,
                   group_by: Optional[str] = None) -> None:
     """A function which will gather all log files within a given folder and pass them along for visualization.
@@ -172,13 +167,11 @@ def parse_log_dir(dir_path: str,
         save_path: Where to save the image if save is true. Defaults to dir_path if not provided.
         ignore_metrics: Any metrics within the log files which will not be visualized.
         include_metrics: A whitelist of metric keys (None whitelists all keys).
-        share_legend: Whether to have one legend across all graphs (True) or one legend per graph (False).
         pretty_names: Whether to modify the metric names in graph titles (True) or leave them alone (False).
         group_by: Combine multiple log files by a regex to visualize their mean+-stddev. For example, to group together
             files like [a_1.txt, a_2.txt] vs [b_1.txt, b_2.txt] you can use: r'(.*)_[\d]+\.txt'.
     """
-    loader = DirDataset(root_dir=dir_path, file_extension=log_extension, recursive_search=recursive_search)
-    file_paths = list(map(lambda d: d['x'], loader.data.values()))
+    file_paths = list_files(root_dir=dir_path, file_extension=log_extension, recursive_search=recursive_search)
 
     parse_log_files(file_paths,
                     log_extension,
@@ -187,6 +180,5 @@ def parse_log_dir(dir_path: str,
                     save_path,
                     ignore_metrics,
                     include_metrics,
-                    share_legend,
                     pretty_names,
                     group_by)
