@@ -12,18 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+import copyreg
 import datetime
 import json
 import os
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Sequence, Tuple, TypeVar, Union
 
 import dill as pickle  # Need to use dill since tf.Variable is a weakref object on multi-gpu machines
 import tensorflow as tf
 import torch
+from tensorflow.python.distribute.mirrored_strategy import MirroredStrategy
 
-from fastestimator.backend.load_model import load_model
-from fastestimator.backend.save_model import save_model
+from fastestimator.backend._load_model import load_model
+from fastestimator.backend._save_model import save_model
 from fastestimator.network import BaseNetwork
 from fastestimator.pipeline import Pipeline
 from fastestimator.schedule.schedule import Scheduler
@@ -36,6 +38,20 @@ if TYPE_CHECKING:
 Model = TypeVar('Model', tf.keras.Model, torch.nn.Module)
 
 
+def pickle_mirroredstrategy(obj: MirroredStrategy) -> Tuple[Callable, Tuple]:
+    """A custom reduce function to use when Pickle encounters a tf MirroredStrategy.
+
+    This relies on the fact that the tf strategy will already be set before the System.load_state method gets called.
+
+    Args:
+        obj: The MirroredStrategy instance.
+
+    Returns:
+        The mechanism to construct a new instance of the MirroredStrategy. See Python docs on the __reduce__ method.
+    """
+    return tf.distribute.get_strategy, ()
+
+
 class System:
     """A class which tracks state information while the fe.Estimator is running.
 
@@ -49,13 +65,16 @@ class System:
         num_devices: How many GPUs are available for training.
         log_steps: Log every n steps (0 to disable train logging, None to disable all logging).
         total_epochs: How many epochs training is expected to run for.
-        max_train_steps_per_epoch: Whether training epochs will be cut short after N steps (or use None if they will run
+        train_steps_per_epoch: Whether training iterations will be cut short or extended to complete N steps (or use None if they will run
             to completion)
+        eval_steps_per_epoch: Whether evaluation iterations will be cut short or extended to complete N steps (or use None if they will run
+            to completion)
+        eval_log_steps: The list of steps on which evaluation progress logs need to be printed.
         system_config: A description of the initialization parameters defining the associated estimator.
 
     Attributes:
         mode: What is the current execution mode of the estimator ('train', 'eval', 'test'), None if warmup.
-        ds_id: The current dataset id, None if there is only one dataset in each mode.
+        ds_id: The current dataset id, Empty string if there is only one dataset in each mode.
         exp_id: A unique identifier for current training experiment.
         global_step: How many training steps have elapsed.
         num_devices: How many GPUs are available for training.
@@ -67,15 +86,18 @@ class System:
         network: A reference to the network being used.
         pipeline: A reference to the pipeline being used.
         traces: The traces being used.
-        max_train_steps_per_epoch: Training will complete after n steps even if loader is not yet exhausted.
-        max_eval_steps_per_epoch: Evaluation will complete after n steps even if loader is not yet exhausted.
+        train_steps_per_epoch: Training will be cut short or extended to complete N steps even if loader is not yet
+            exhausted. If None, all data will be used.
+        eval_steps_per_epoch: Evaluation will be cut short or extended to complete N steps even if loader is not yet
+            exhausted. If None, all data will be used.
+        eval_log_steps: The list of steps on which evaluation progress logs need to be printed.
         summary: An object to write experiment results to.
         experiment_time: A timestamp indicating when this model was trained.
         custom_graphs: A place to store extra graphs which are too complicated for the primary history.
     """
 
     mode: Optional[str]
-    ds_id: Optional[str]
+    ds_id: str
     exp_id: int
     global_step: Optional[int]
     num_devices: int
@@ -87,8 +109,9 @@ class System:
     network: BaseNetwork
     pipeline: Pipeline
     traces: List[Union['Trace', Scheduler['Trace']]]
-    max_train_steps_per_epoch: Optional[int]
-    max_eval_steps_per_epoch: Optional[int]
+    train_steps_per_epoch: Optional[int]
+    eval_steps_per_epoch: Optional[int]
+    eval_log_steps: Sequence[int]
     summary: Summary
     experiment_time: str
     custom_graphs: Dict[str, List[Summary]]
@@ -98,16 +121,18 @@ class System:
                  pipeline: Pipeline,
                  traces: List[Union['Trace', Scheduler['Trace']]],
                  mode: Optional[str] = None,
-                 ds_id: Optional[str] = None,
+                 ds_id: str = '',
                  num_devices: int = torch.cuda.device_count(),
                  log_steps: Optional[int] = None,
                  total_epochs: int = 0,
-                 max_train_steps_per_epoch: Optional[int] = None,
-                 max_eval_steps_per_epoch: Optional[int] = None,
+                 train_steps_per_epoch: Optional[int] = None,
+                 eval_steps_per_epoch: Optional[int] = None,
+                 eval_log_steps: Sequence[int] = (),
                  system_config: Optional[List[FeSummaryTable]] = None) -> None:
 
         self.network = network
         self.pipeline = pipeline
+        self.eval_log_steps = eval_log_steps
         self.traces = traces
         self.mode = mode
         self.ds_id = ds_id
@@ -115,14 +140,23 @@ class System:
         self.log_steps = log_steps
         self.total_epochs = total_epochs
         self.batch_idx = None
-        self.max_train_steps_per_epoch = max_train_steps_per_epoch
-        self.max_eval_steps_per_epoch = max_eval_steps_per_epoch
+        self.train_steps_per_epoch = train_steps_per_epoch
+        self.eval_steps_per_epoch = eval_steps_per_epoch
         self.stop_training = False
         self.summary = Summary(None, system_config)
         self.experiment_time = ""
         self.custom_graphs = {}
         self._initialize_state()
 
+    @property
+    def steps_per_epoch(self) -> Optional[int]:
+        if self.mode == 'train':
+            return self.train_steps_per_epoch
+        elif self.mode == 'eval':
+            return self.eval_steps_per_epoch
+        else:
+            return None
+
     def _initialize_state(self) -> None:
         """Initialize the training state.
         """
@@ -156,7 +190,7 @@ class System:
         """
         self.experiment_time = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
         self.mode = "train"
-        self.ds_id = None
+        self.ds_id = ''
         self._initialize_state()
         self.batch_idx = None
         self.stop_training = False
@@ -171,7 +205,7 @@ class System:
         """
         self.experiment_time = self.experiment_time or datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
         self.mode = "test"
-        self.ds_id = None
+        self.ds_id = ''
         if not self.stop_training:
             self.epoch_idx = self.total_epochs
         self.stop_training = False
@@ -235,7 +269,12 @@ class System:
             }
         }
         with open(os.path.join(save_dir, 'objects.pkl'), 'wb') as file:
-            pickle.dump(objects, file)
+            # We need to use a custom pickler here to handle MirroredStrategy, which will show up inside of tf
+            # MirroredVariables in multi-gpu systems.
+            p = pickle.Pickler(file)
+            p.dispatch_table = copyreg.dispatch_table.copy()
+            p.dispatch_table[MirroredStrategy] = pickle_mirroredstrategy
+            p.dump(objects)
 
     def load_state(self, load_dir: str) -> None:
         """Load training state.
@@ -350,7 +389,7 @@ class System:
             elif hasattr(obj, '__dict__'):
                 obj.__dict__.update(state)
             elif isinstance(obj, dict):
-                [System._load_dict(states, k, obj) for k in states]
+                System._load_dict(states, key, obj)
             else:
                 # Might be a None or something else that can't be updated
                 pass
