@@ -14,15 +14,12 @@
 # ==============================================================================
 from typing import Sequence, Union, Iterable
 
-import matplotlib.pyplot as plt
-
 from fastestimator.summary.logs.log_plot import visualize_logs
 from fastestimator.summary.summary import Summary
 from fastestimator.trace.trace import Trace
 from fastestimator.util.data import Data
-from fastestimator.util.img_data import ImgData
+from fastestimator.util.img_data import Display, ImageDisplay
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import show_image
 
 
 @traceable()
@@ -36,17 +33,15 @@ class ImageViewer(Trace):
         mode: What mode(s) to execute this Trace in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
-        width: The width in inches of the figure.
-        height: The height in inches of the figure.
+        interactive: Whether the image should be interactive. This is False by default to reduce jupyter file size.
     """
-    def __init__(
-        self,
-        inputs: Union[str, Sequence[str]],
-        mode: Union[None, str, Iterable[str]] = ("eval", "test"),
-        width: int = 12,
-        height: int = 6) -> None:
+    def __init__(self,
+                 inputs: Union[str, Sequence[str]],
+                 mode: Union[None, str, Iterable[str]] = ("eval", "test"),
+                 interactive: bool = False,
+                 ) -> None:
         super().__init__(inputs=inputs, mode=mode)
-        plt.rcParams['figure.figsize'] = [width, height]
+        self.interactive = interactive
 
     def on_epoch_end(self, data: Data) -> None:
         self._display_images(data)
@@ -63,17 +58,13 @@ class ImageViewer(Trace):
         for key in self.inputs:
             if key in data:
                 imgs = data[key]
-                if isinstance(imgs, ImgData):
-                    fig = imgs.paint_numpy(dpi=96)
-                    plt.imshow(fig[0])
-                    plt.axis('off')
-                    plt.tight_layout()
-                    plt.show()
+                if isinstance(imgs, Display):
+                    imgs.show(interactive=self.interactive)
                 elif isinstance(imgs, Summary):
                     visualize_logs([imgs])
                 elif isinstance(imgs, (list, tuple)) and all([isinstance(img, Summary) for img in imgs]):
                     visualize_logs(imgs)
                 else:
                     for idx, img in enumerate(imgs):
-                        show_image(img, title="{}_{}".format(key, idx))
-                        plt.show()
+                        fig = ImageDisplay(image=img, title="{}_{}".format(key, idx))
+                        fig.show(interactive=self.interactive)
