@@ -17,29 +17,33 @@ import re
 from collections import defaultdict, namedtuple
 from typing import Any, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
 
-import matplotlib.pyplot as plt
+import cv2
+import numpy as np
 import tensorboard as tb
 import tensorflow as tf
 import torch
+from plotly.graph_objs import Figure
+from tensorflow.keras import backend
 from tensorflow.python.framework import ops as tfops
-from tensorflow.python.keras import backend
+from tensorflow.python.keras.callbacks import keras_model_summary
 from tensorflow.python.ops import summary_ops_v2
 from torch.utils.tensorboard import SummaryWriter
 
-from fastestimator.backend.abs import abs
-from fastestimator.backend.concat import concat
-from fastestimator.backend.expand_dims import expand_dims
-from fastestimator.backend.permute import permute
-from fastestimator.backend.reduce_sum import reduce_sum
-from fastestimator.backend.reshape import reshape
-from fastestimator.backend.squeeze import squeeze
-from fastestimator.backend.to_tensor import to_tensor
+from fastestimator.backend._abs import abs
+from fastestimator.backend._concat import concat
+from fastestimator.backend._expand_dims import expand_dims
+from fastestimator.backend._permute import permute
+from fastestimator.backend._reduce_sum import reduce_sum
+from fastestimator.backend._reshape import reshape
+from fastestimator.backend._squeeze import squeeze
+from fastestimator.backend._to_tensor import to_tensor
 from fastestimator.network import BaseNetwork, TFNetwork
 from fastestimator.trace.trace import Trace
+from fastestimator.util.base_util import to_set, to_list, is_number, DefaultKeyDict
 from fastestimator.util.data import Data
-from fastestimator.util.img_data import ImgData
+from fastestimator.util.img_data import Display
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import DefaultKeyDict, is_number, to_list, to_number, to_set
+from fastestimator.util.util import to_number
 
 # https://github.com/pytorch/pytorch/issues/30966
 tf.io.gfile = tb.compat.tensorflow_stub.io.gfile
@@ -107,10 +111,13 @@ class _BaseWriter:
             step: The current training step.
         """
         for key, img in images:
-            if isinstance(img, ImgData):
-                img = img.paint_figure()
-            if isinstance(img, plt.Figure):
-                self.summary_writers[mode].add_figure(tag=key, figure=img, global_step=step)
+            if isinstance(img, Display):
+                img = img.prepare()
+            if isinstance(img, Figure):
+                img = img.to_image(format='png')
+                img = cv2.imdecode(np.frombuffer(img, np.uint8), cv2.IMREAD_COLOR)
+                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+                self.summary_writers[mode].add_image(tag=key, img_tensor=img, global_step=step, dataformats='HWC')
             else:
                 self.summary_writers[mode].add_images(tag=key,
                                                       img_tensor=to_number(img),
@@ -207,13 +214,13 @@ class _TfWriter(_BaseWriter):
             # Record the overall execution summary
             if hasattr(self.network._forward_step_static, '_concrete_stateful_fn'):
                 # noinspection PyProtectedMember
-                summary_ops_v2.graph(self.network._forward_step_static._concrete_stateful_fn.graph, step=epoch)
+                summary_ops_v2.graph(self.network._forward_step_static._concrete_stateful_fn.graph)
             # Record the individual model summaries
             for model in self.network.epoch_models:
                 summary_writable = (model.__class__.__name__ == 'Sequential'
                                     or (hasattr(model, '_is_graph_network') and model._is_graph_network))
                 if summary_writable:
-                    summary_ops_v2.keras_model(model.model_name, model, step=epoch)
+                    keras_model_summary(model.model_name, model, step=epoch)
 
     def write_weights(self, mode: str, models: Iterable[Model], step: int, visualize: bool) -> None:
         # Similar to TF implementation, but multiple models
@@ -297,6 +304,8 @@ class TensorBoard(Trace):
     Freq = namedtuple('Freq', ['is_step', 'freq'])
     writer: _BaseWriter
 
+    # TODO - support for per-instance tracking
+
     def __init__(self,
                  log_dir: str = 'logs',
                  update_freq: Union[None, int, str] = 100,
@@ -309,7 +318,7 @@ class TensorBoard(Trace):
                  embedding_labels: Union[None, str, List[str]] = None,
                  embedding_images: Union[None, str, List[str]] = None) -> None:
         super().__init__(inputs=["*"] + to_list(write_images) + to_list(write_embeddings) + to_list(embedding_labels) +
-                         to_list(embedding_images))
+                                to_list(embedding_images))
         self.root_log_dir = log_dir
         self.update_freq = self._parse_freq(update_freq)
         self.write_graph = write_graph
@@ -342,8 +351,9 @@ class TensorBoard(Trace):
         else:
             embedding_images = [None for _ in range(len(write_embeddings))]
         self.write_embeddings = [(feature, label, img_label) for feature,
-                                 label,
-                                 img_label in zip(write_embeddings, embedding_labels, embedding_images)]
+                                                                 label,
+                                                                 img_label in
+                                 zip(write_embeddings, embedding_labels, embedding_images)]
         self.collected_embeddings = defaultdict(list)
 
     def _parse_freq(self, freq: Union[None, str, int]) -> Freq:
@@ -382,7 +392,7 @@ class TensorBoard(Trace):
             os.path.abspath(os.path.join(self.root_log_dir, self.system.experiment_time))))
         self.writer = _TfWriter(self.root_log_dir, self.system.experiment_time, self.system.network) if isinstance(
             self.system.network, TFNetwork) else _TorchWriter(
-                self.root_log_dir, self.system.experiment_time, self.system.network)
+            self.root_log_dir, self.system.experiment_time, self.system.network)
         if self.write_graph and self.system.global_step == 1:
             self.painted_graphs = set()
 
