@@ -16,7 +16,7 @@ from typing import Iterable, Union
 
 import numpy as np
 
-from fastestimator.trace.meta.per_ds import per_ds
+from fastestimator.trace.meta._per_ds import per_ds
 from fastestimator.trace.trace import Trace
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
@@ -74,7 +74,7 @@ class Accuracy(Trace):
         y_true, y_pred = to_number(data[self.true_key]), to_number(data[self.pred_key])
         if y_true.shape[-1] > 1 and y_true.ndim > 1:
             y_true = np.argmax(y_true, axis=-1)
-        if y_pred.shape[-1] > 1:
+        if y_pred.shape[-1] > 1 and y_pred.ndim > 1:
             y_pred = np.argmax(y_pred, axis=-1)
         else:  # binaray classification (pred shape is [batch, 1])
             if self.from_logits:
@@ -83,6 +83,7 @@ class Accuracy(Trace):
         assert y_pred.size == y_true.size
         self.correct += np.sum(y_pred.ravel() == y_true.ravel())
         self.total += len(y_pred.ravel())
+        data.write_per_instance_log(self.outputs[0], np.array(y_pred.ravel() == y_true.ravel(), dtype=np.int8))
 
     def on_epoch_end(self, data: Data) -> None:
         data.write_with_log(self.outputs[0], self.correct / self.total)
