@@ -12,11 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from typing import Union, Iterable
+from typing import Iterable, Union
 
 import numpy as np
 
-from fastestimator.trace.meta.per_ds import per_ds
+from fastestimator.backend._dice_score import dice_score
+from fastestimator.trace.meta._per_ds import per_ds
 from fastestimator.trace.trace import Trace
 from fastestimator.util import Data
 from fastestimator.util.traceability_util import traceable
@@ -32,6 +33,7 @@ class Dice(Trace):
         true_key: The key of the ground truth mask.
         pred_key: The key of the prediction values.
         threshold: The threshold for binarizing the prediction.
+        channel_average: Whether the average channel-wise dice loss.
         mode: What mode(s) to execute this Trace in. For example, "train", "eval", "test", or "infer". To execute
             regardless of mode, pass None. To execute in all modes except for a particular one, you can pass an argument
             like "!infer" or "!train".
@@ -42,17 +44,21 @@ class Dice(Trace):
             computing an aggregate across all ds_ids on which it runs. This is automatically False if `output_name`
             contains a "|" character.
     """
+
     def __init__(self,
                  true_key: str,
                  pred_key: str,
                  threshold: float = 0.5,
+                 channel_average: bool = False,
                  mode: Union[None, str, Iterable[str]] = ("eval", "test"),
                  ds_id: Union[None, str, Iterable[str]] = None,
                  output_name: str = "Dice",
                  per_ds: bool = True) -> None:
-        super().__init__(inputs=(true_key, pred_key), mode=mode, outputs=output_name, ds_id=ds_id)
+        super().__init__(inputs=(true_key, pred_key),
+                         mode=mode, outputs=output_name, ds_id=ds_id)
         self.threshold = threshold
         self.smooth = 1e-8
+        self.channel_average = channel_average
         self.dice = []
         self.per_ds = per_ds
 
@@ -68,15 +74,16 @@ class Dice(Trace):
         self.dice = []
 
     def on_batch_end(self, data: Data) -> None:
-        y_true, y_pred = to_number(data[self.true_key]), to_number(data[self.pred_key])
-        batch_size = y_true.shape[0]
-        y_true, y_pred = y_true.reshape((batch_size, -1)), y_pred.reshape((batch_size, -1))
+        y_true, y_pred = to_number(
+            data[self.true_key]), to_number(data[self.pred_key])
 
-        prediction_label = (y_pred >= self.threshold).astype(np.int32)
+        y_pred = np.where(y_pred > self.threshold, 1.0,
+                          0.0).astype(y_pred.dtype)
 
-        intersection = np.sum(y_true * prediction_label, axis=-1)
-        area_sum = np.sum(y_true, axis=-1) + np.sum(prediction_label, axis=-1)
-        dice = (2. * intersection + self.smooth) / (area_sum + self.smooth)
+        dice = dice_score(y_pred=y_pred, y_true=y_true,
+                          channel_average=self.channel_average, epsilon=self.smooth)
+
+        data.write_per_instance_log(self.outputs[0], dice)
         self.dice.extend(list(dice))
 
     def on_epoch_end(self, data: Data) -> None:
