@@ -19,7 +19,7 @@ from typing import Dict, List, Union, Iterable
 import numpy as np
 from pycocotools import mask as maskUtils
 
-from fastestimator.trace.meta.per_ds import per_ds
+from fastestimator.trace.meta._per_ds import per_ds
 from fastestimator.trace.trace import Trace
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
@@ -375,7 +375,7 @@ class MeanAveragePrecision(Trace):
                     for recall_index, precision_index in enumerate(inds):
                         precision_at_recall[recall_index] = precision[precision_index]
                         score[recall_index] = det_scores_sorted[precision_index]
-                except:
+                except IndexError:
                     pass
 
                 precision_matrix[index, :, cat_index] = np.array(precision_at_recall)
