@@ -17,7 +17,7 @@ from typing import Any, Dict, Union, Iterable
 import numpy as np
 from sklearn.metrics import recall_score
 
-from fastestimator.trace.meta.per_ds import per_ds
+from fastestimator.trace.meta._per_ds import per_ds
 from fastestimator.trace.trace import Trace
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
@@ -79,7 +79,7 @@ class Recall(Trace):
         self.binary_classification = y_pred.shape[-1] == 1
         if y_true.shape[-1] > 1 and y_true.ndim > 1:
             y_true = np.argmax(y_true, axis=-1)
-        if y_pred.shape[-1] > 1:
+        if y_pred.shape[-1] > 1 and y_pred.ndim > 1:
             y_pred = np.argmax(y_pred, axis=-1)
         else:
             y_pred = np.round(y_pred)
