@@ -13,18 +13,19 @@
 # limitations under the License.
 # ==============================================================================
 import time
-from collections import deque
+from collections import defaultdict, deque
 from typing import Iterable, List, Set, Union
 
 import numpy as np
 from natsort import humansorted
 
-from fastestimator.backend.get_lr import get_lr
+from fastestimator.backend._get_lr import get_lr
 from fastestimator.summary.summary import ValWithError
 from fastestimator.summary.system import System
-from fastestimator.util.data import Data
+from fastestimator.util.base_util import check_ds_id, check_io_names, parse_modes, to_list, to_set
+from fastestimator.util.data import Data, DSData
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import check_ds_id, check_io_names, parse_modes, to_list, to_number, to_set
+from fastestimator.util.util import to_number
 
 
 @traceable()
@@ -160,6 +161,7 @@ class TrainEssential(Trace):
     Args:
         monitor_names: Which keys from the data dictionary to monitor during training.
     """
+
     def __init__(self, monitor_names: Set[str]) -> None:
         super().__init__(inputs=monitor_names, mode="train", outputs=["steps/sec", "epoch_time", "total_time"])
         self.elapse_times = []
@@ -180,6 +182,8 @@ class TrainEssential(Trace):
     def on_batch_end(self, data: Data) -> None:
         if self.system.log_steps and (self.system.global_step % self.system.log_steps == 0
                                       or self.system.global_step == 1):
+            if self.system.ds_id != '':
+                data = DSData(self.system.ds_id, data)
             for key in self.inputs:
                 if key in data:
                     data.write_with_log(key, data[key])
@@ -211,24 +215,41 @@ class EvalEssential(Trace):
     Args:
         monitor_names: Any keys which should be collected over the course of an eval epoch.
     """
+
     def __init__(self, monitor_names: Set[str]) -> None:
-        super().__init__(mode="eval", inputs=monitor_names)
-        self.eval_results = None
+        super().__init__(mode="eval", inputs=monitor_names, outputs=["steps/sec"])
+        self.step_start = time.perf_counter()
+        self.eval_results = defaultdict(lambda: defaultdict(list))
 
     def on_epoch_begin(self, data: Data) -> None:
-        self.eval_results = None
+        self.eval_results = defaultdict(lambda: defaultdict(list))
+        self.eval_step = 0
+        self.elapsed_step = 0
+        self.step_start = time.perf_counter()
+
+    def on_batch_begin(self, data: Data) -> None:
+        self.eval_step += 1
 
     def on_batch_end(self, data: Data) -> None:
-        if self.eval_results is None:
-            self.eval_results = {key: [data[key]] for key in self.inputs if key in data}
-        else:
-            for key in self.inputs:
-                if key in data:
-                    self.eval_results[key].append(data[key])
+        for key in self.inputs:
+            if key in data:
+                self.eval_results[key][self.system.ds_id].append(data[key])
+
+        if self.system.mode == "eval" and self.eval_step in self.system.eval_log_steps:
+            if self.eval_step > 1:
+                elapsed_time = time.perf_counter() - self.step_start
+                elapsed_step = self.eval_step - self.elapsed_step
+                data.write_with_log("steps/sec", round(elapsed_step / elapsed_time, 2))
+            self.elapsed_step = self.eval_step
+            self.step_start = time.perf_counter()
 
     def on_epoch_end(self, data: Data) -> None:
-        for key, value_list in self.eval_results.items():
-            data.write_with_log(key, np.mean(np.array(value_list), axis=0))
+        for key, ds_vals in self.eval_results.items():
+            for ds_id, vals in ds_vals.items():
+                if ds_id != '':
+                    d = DSData(ds_id, data)
+                    d.write_with_log(key, np.mean(np.array(vals), axis=0))
+            data.write_with_log(key, np.mean(np.array([e for x in ds_vals.values() for e in x]), axis=0))
 
 
 @traceable()
@@ -240,24 +261,26 @@ class TestEssential(Trace):
     Args:
         monitor_names: Any keys which should be collected over the course of an test epoch.
     """
+
     def __init__(self, monitor_names: Set[str]) -> None:
         super().__init__(mode="test", inputs=monitor_names)
-        self.test_results = None
+        self.test_results = defaultdict(lambda: defaultdict(list))
 
     def on_epoch_begin(self, data: Data) -> None:
-        self.test_results = None
+        self.test_results = defaultdict(lambda: defaultdict(list))
 
     def on_batch_end(self, data: Data) -> None:
-        if self.test_results is None:
-            self.test_results = {key: [data[key]] for key in self.inputs if key in data}
-        else:
-            for key in self.inputs:
-                if key in data:
-                    self.test_results[key].append(data[key])
+        for key in self.inputs:
+            if key in data:
+                self.test_results[key][self.system.ds_id].append(data[key])
 
     def on_epoch_end(self, data: Data) -> None:
-        for key, value_list in self.test_results.items():
-            data.write_with_log(key, np.mean(np.array(value_list), axis=0))
+        for key, ds_vals in self.test_results.items():
+            for ds_id, vals in ds_vals.items():
+                if ds_id != '':
+                    d = DSData(ds_id, data)
+                    d.write_with_log(key, np.mean(np.array(vals), axis=0))
+            data.write_with_log(key, np.mean(np.array([e for x in ds_vals.values() for e in x]), axis=0))
 
 
 @traceable()
@@ -266,6 +289,7 @@ class Logger(Trace):
 
     Please don't add this trace into an estimator manually. FastEstimator will add it automatically.
     """
+
     def __init__(self) -> None:
         super().__init__(inputs="*")
 
@@ -274,15 +298,25 @@ class Logger(Trace):
             start_step = 1 if not self.system.global_step else self.system.global_step
             self._print_message("FastEstimator-Start: step: {}; ".format(start_step), data)
 
+    def on_epoch_begin(self, data: Data) -> None:
+        if self.system.mode == 'eval':
+            self.eval_step = 0
+
     def on_batch_end(self, data: Data) -> None:
         if self.system.mode == "train" and self.system.log_steps and (self.system.global_step % self.system.log_steps
                                                                       == 0 or self.system.global_step == 1):
             self._print_message("FastEstimator-Train: step: {}; ".format(self.system.global_step), data)
 
+        if self.system.mode == "eval":
+            self.eval_step += 1
+            if self.eval_step in self.system.eval_log_steps:
+                self._print_message("Eval Progress: {}/{}; ".format(self.eval_step, self.system.eval_log_steps[-1]),
+                                    data)
+
     def on_epoch_end(self, data: Data) -> None:
-        if self.system.mode == "train" and self.system.log_steps:
+        if self.system.mode == "train":
             self._print_message("FastEstimator-Train: step: {}; ".format(self.system.global_step), data, True)
-        elif self.system.mode == "eval":
+        elif self.system.mode == 'eval':
             self._print_message("FastEstimator-Eval: step: {}; ".format(self.system.global_step), data, True)
         elif self.system.mode == "test":
             self._print_message("FastEstimator-Test: step: {}; ".format(self.system.global_step), data, True)
@@ -396,6 +430,7 @@ def sort_traces(traces: List[Trace], ds_ids: List[str], available_outputs: Union
 
 
 class PerDSTrace(Trace):
+
     def on_ds_begin(self, data: Data) -> None:
         """Runs at the beginning of each dataset.
 
