@@ -19,7 +19,7 @@ import numpy as np
 
 from fastestimator.trace.io import ImageViewer
 from fastestimator.util.data import Data
-from fastestimator.util.img_data import ImgData
+from fastestimator.util.img_data import BatchDisplay, GridDisplay
 
 
 class TestImageViewer(unittest.TestCase):
@@ -29,16 +29,17 @@ class TestImageViewer(unittest.TestCase):
         mask = np.zeros_like(cls.image)
         mask[0, 10:20, 10:30, :] = [1, 0, 0]
         bbox = np.array([[[3, 7, 10, 6, 'box1'], [20, 20, 8, 8, 'box2']]] * 1)
-        cls.img_data = ImgData(y=np.ones((4, )), x=[cls.image, mask, bbox])
+        cls.img_data = GridDisplay([BatchDisplay(text=np.ones((1, )), title="y"),
+                                    BatchDisplay(image=cls.image, masks=mask, bboxes=bbox, title='x')])
 
-    @patch("matplotlib.pyplot.show")
+    @patch("fastestimator.util.base_util.FigureFE.show")
     def test_image_on_epoch_end(self, mock_show):
         data = Data({'x': 0.5 * np.ones(shape=(1, 28, 28, 3))})
         imageviewer = ImageViewer(inputs='x')
         imageviewer.on_epoch_end(data=data)
         self.assertEqual(len(mock_show.mock_calls), 1)
 
-    @patch("matplotlib.pyplot.show")
+    @patch("fastestimator.util.base_util.FigureFE.show")
     def test_imgdata_on_epoch_end(self, mock_show):
         data = Data({'x': self.img_data})
         imageviewer = ImageViewer(inputs='x')
