@@ -12,29 +12,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+import json
 import os
-import pickle
 import unittest
 import urllib.request
+from urllib.error import URLError, HTTPError, ContentTooShortError
 
 
-def ping(host):
+def ping(host: str):
     try:
+        assert host.startswith('http')
         agent = 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.76 Safari/537.36'
         req = urllib.request.Request(host)
         req.add_header('User-Agent', agent)
         code = urllib.request.urlopen(req).getcode()
         return code == 200
-    except:
+    except (ValueError, URLError, HTTPError, ContentTooShortError):
         return False
 
 
 class TestData(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        urls_dict_path = os.path.abspath(os.path.join(__file__, "..", "resources", "dataset_urls.pickle"))
+        urls_dict_path = os.path.abspath(os.path.join(__file__, "..", "resources", "dataset_urls.json"))
         with open(urls_dict_path, 'rb') as dataset_url_dict:
-            cls.data_urls = pickle.load(dataset_url_dict)
+            cls.data_urls = json.load(dataset_url_dict)
 
     def test_dataset_urls(self):
         failed_urls = set()
