@@ -36,5 +36,4 @@ class TestLambdaOp(unittest.TestCase):
         op = LambdaOp(fn=np.sum)
         data = tf.convert_to_tensor([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
         result = op.forward_batch(data=[data], state={})
-        ans = np.array([6, 15, 24], dtype=np.float32)
-        self.assertTrue(np.array_equal(result, ans))
+        self.assertEqual(result, 45)
